import { effectScope, onScopeDispose } from 'vue'
import { useIntervalFn } from '@vueuse/core'
import uesFetch from '@/hooks/useFetch'

import type { EffectScope, ShallowRef } from 'vue'
import type { ExtractApiResponse } from 'api-type'
import type {} from 'global-type'

const useMessageLoop = () => {
  const { fetch: getUnReadMessage, result: unReadMessage } = uesFetch('getUnReadMessage')
  useIntervalFn(getUnReadMessage, 10 * 1000)

  return unReadMessage
}

function createSharedComposable(composable: typeof useMessageLoop) {
  let subscribers = 0
  let state: ShallowRef<ExtractApiResponse<'getUnReadMessage'>> | null | undefined
  let scope: EffectScope | null

  const dispose = () => {
    if (scope && --subscribers <= 0) {
      scope.stop()
      state = scope = null
    }
  }

  return () => {
    subscribers++
    if (!state) {
      scope = effectScope(true)
      state = scope.run<ShallowRef<ExtractApiResponse<'getUnReadMessage'>>>(composable)
    }
    onScopeDispose(dispose)
    return state
  }
}

export default createSharedComposable(useMessageLoop)
